@@ -31,30 +31,20 @@
 #include "gtktextbuffer.h"
 #include "gtkselectionprivate.h"
 #include "gtkquartz.h"
-#include "quartz/gdkquartz-gtk-only.h"
+
 
 enum {
   OWNER_CHANGE,
   LAST_SIGNAL
 };
 
-@interface GtkClipboardOwner : NSObject {
-  GtkClipboard *clipboard;
-  @public
-  gboolean setting_same_owner;
-}
-
-@end
-
 typedef struct _GtkClipboardClass GtkClipboardClass;
 
-struct _GtkClipboard
+struct _GtkClipboard 
 {
   GObject parent_instance;
 
   NSPasteboard *pasteboard;
-  GtkClipboardOwner *owner;
-  NSInteger change_count;
 
   GdkAtom selection;
 
@@ -86,15 +76,16 @@ struct _GtkClipboardClass
 			GdkEventOwnerChange *event);
 };
 
-static void gtk_clipboard_class_init   (GtkClipboardClass   *class);
-static void gtk_clipboard_finalize     (GObject             *object);
-static void gtk_clipboard_owner_change (GtkClipboard        *clipboard,
-					GdkEventOwnerChange *event);
+@interface GtkClipboardOwner : NSObject {
+  GtkClipboard *clipboard;
 
-static void          clipboard_unset      (GtkClipboard     *clipboard);
-static GtkClipboard *clipboard_peek       (GdkDisplay       *display,
-					   GdkAtom           selection,
-					   gboolean          only_if_exists);
+  GtkClipboardGetFunc get_func;
+  GtkClipboardClearFunc clear_func;
+  gpointer user_data;
+  
+}
+
+@end
 
 @implementation GtkClipboardOwner
 -(void)pasteboard:(NSPasteboard *)sender provideDataForType:(NSString *)type
@@ -108,7 +99,7 @@ static GtkClipboard *clipboard_peek       (GdkDisplay       *display,
   memset (&selection_data, 0, sizeof (GtkSelectionData));
 
   selection_data.selection = clipboard->selection;
-  selection_data.target = gdk_quartz_pasteboard_type_to_atom_libgtk_only (type);
+  selection_data.target = _gtk_quartz_pasteboard_type_to_atom (type);
   selection_data.display = gdk_display_get_default ();
   selection_data.length = -1;
 
@@ -126,26 +117,21 @@ static GtkClipboard *clipboard_peek       (GdkDisplay       *display,
     }
 }
 
-/*  pasteboardChangedOwner is not called immediately, and it's not called
- *  reliably. It is somehow documented in the apple api docs, but the docs
- *  suck and don’t really give clear instructions. Therefore we track
- *  changeCount in several places below and clear the clipboard if it
- *  changed.
- */
 - (void)pasteboardChangedOwner:(NSPasteboard *)sender
 {
-  if (! setting_same_owner)
-    clipboard_unset (clipboard);
+  if (clear_func)
+    clear_func (clipboard, user_data);
+
+  [self release];
 }
 
 - (id)initWithClipboard:(GtkClipboard *)aClipboard
 {
   self = [super init];
 
-  if (self)
+  if (self) 
     {
       clipboard = aClipboard;
-      setting_same_owner = FALSE;
     }
 
   return self;
@@ -153,6 +139,15 @@ static GtkClipboard *clipboard_peek       (GdkDisplay       *display,
 
 @end
 
+static void gtk_clipboard_class_init   (GtkClipboardClass   *class);
+static void gtk_clipboard_finalize     (GObject             *object);
+static void gtk_clipboard_owner_change (GtkClipboard        *clipboard,
+					GdkEventOwnerChange *event);
+
+static void          clipboard_unset      (GtkClipboard     *clipboard);
+static GtkClipboard *clipboard_peek       (GdkDisplay       *display,
+					   GdkAtom           selection,
+					   gboolean          only_if_exists);
 
 static const gchar clipboards_owned_key[] = "gtk-clipboards-owned";
 static GQuark clipboards_owned_key_id = 0;
@@ -164,7 +159,7 @@ GType
 gtk_clipboard_get_type (void)
 {
   static GType clipboard_type = 0;
-
+  
   if (!clipboard_type)
     {
       const GTypeInfo clipboard_info =
@@ -179,11 +174,11 @@ gtk_clipboard_get_type (void)
 	0,              /* n_preallocs */
 	(GInstanceInitFunc) NULL,
       };
-
+      
       clipboard_type = g_type_register_static (G_TYPE_OBJECT, I_("GtkClipboard"),
 					       &clipboard_info, 0);
     }
-
+  
   return clipboard_type;
 }
 
@@ -193,23 +188,18 @@ gtk_clipboard_class_init (GtkClipboardClass *class)
   GObjectClass *gobject_class = G_OBJECT_CLASS (class);
 
   parent_class = g_type_class_peek_parent (class);
-
+  
   gobject_class->finalize = gtk_clipboard_finalize;
 
   class->owner_change = gtk_clipboard_owner_change;
 
-  /**
-   * GtkClipboard::owner-change:
-   * @clipboard:
-   * @event: (type GdkEventOwnerChange):
-   */
   clipboard_signals[OWNER_CHANGE] =
     g_signal_new (I_("owner-change"),
 		  G_TYPE_FROM_CLASS (gobject_class),
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (GtkClipboardClass, owner_change),
 		  NULL, NULL,
-		  NULL,
+		  _gtk_marshal_VOID__BOXED,
 		  G_TYPE_NONE, 1,
 		  GDK_TYPE_EVENT | G_SIGNAL_TYPE_STATIC_SCOPE);
 }
@@ -227,7 +217,7 @@ gtk_clipboard_finalize (GObject *object)
     g_warning ("GtkClipboard prematurely finalized");
 
   clipboard_unset (clipboard);
-
+  
   clipboards = g_object_get_data (G_OBJECT (clipboard->display), "gtk-clipboard-list");
   clipboards = g_slist_remove (clipboards, clipboard);
   g_object_set_data (G_OBJECT (clipboard->display), I_("gtk-clipboard-list"), clipboards);
@@ -257,13 +247,6 @@ clipboard_display_closed (GdkDisplay   *display,
   g_object_unref (clipboard);
 }
 
-/**
- * gtk_clipboard_get_for_display:
- * @display:
- * @selection:
- *
- * Returns: (transfer none):
- */
 GtkClipboard *
 gtk_clipboard_get_for_display (GdkDisplay *display,
 			       GdkAtom     selection)
@@ -274,38 +257,12 @@ gtk_clipboard_get_for_display (GdkDisplay *display,
   return clipboard_peek (display, selection, FALSE);
 }
 
-/**
- * gtk_clipboard_get:
- * @selection:
- *
- * Returns: (transfer none):
- */
 GtkClipboard *
 gtk_clipboard_get (GdkAtom selection)
 {
   return gtk_clipboard_get_for_display (gdk_display_get_default (), selection);
 }
 
-/**
- * gtk_clipboard_get_default:
- * @display: the #GdkDisplay for which the clipboard is to be retrieved.
- *
- * Returns the default clipboard object for use with cut/copy/paste menu items
- * and keyboard shortcuts.
- *
- * Return value: (transfer none): the default clipboard object.
- *
- * Since: 3.16
- **/
-GtkClipboard *
-gtk_clipboard_get_default (GdkDisplay *display)
-{
-  g_return_val_if_fail (display != NULL, NULL);
-  g_return_val_if_fail (GDK_IS_DISPLAY (display), NULL);
-
-  return gtk_clipboard_get_for_display (display, GDK_SELECTION_CLIPBOARD);
-}
-
 static void
 clipboard_owner_destroyed (gpointer data)
 {
@@ -332,7 +289,7 @@ clipboard_owner_destroyed (gpointer data)
 
       tmp_list = tmp_list->next;
     }
-
+  
   g_slist_free (clipboards);
 }
 
@@ -341,7 +298,7 @@ clipboard_add_owner_notify (GtkClipboard *clipboard)
 {
   if (!clipboards_owned_key_id)
     clipboards_owned_key_id = g_quark_from_static_string (clipboards_owned_key);
-
+  
   if (clipboard->have_owner)
     g_object_set_qdata_full (clipboard->user_data, clipboards_owned_key_id,
 			     g_slist_prepend (g_object_steal_qdata (clipboard->user_data,
@@ -370,9 +327,14 @@ gtk_clipboard_set_contents (GtkClipboard         *clipboard,
 			    gpointer              user_data,
 			    gboolean              have_owner)
 {
+  GtkClipboardOwner *owner;
   NSSet *types;
   NSAutoreleasePool *pool;
 
+  pool = [[NSAutoreleasePool alloc] init];
+
+  owner = [[GtkClipboardOwner alloc] initWithClipboard:clipboard];
+
   if (!(clipboard->have_owner && have_owner) ||
       clipboard->user_data != user_data)
     {
@@ -387,53 +349,17 @@ gtk_clipboard_set_contents (GtkClipboard         *clipboard,
               clipboard->user_data != user_data)
             {
               (*clear_func) (clipboard, user_data);
+              [pool release];
               return FALSE;
             }
           else
             {
+              [pool release];
               return TRUE;
             }
         }
     }
 
-  pool = [[NSAutoreleasePool alloc] init];
-
-  types = _gtk_quartz_target_entries_to_pasteboard_types (targets, n_targets);
-
-  /*  call declareTypes before setting the clipboard members because
-   *  declareTypes might clear the clipboard
-   */
-  if (user_data && user_data == clipboard->user_data)
-    {
-      clipboard->owner->setting_same_owner = TRUE;
-      clipboard->change_count = [clipboard->pasteboard declareTypes: [types allObjects]
-                                                              owner: clipboard->owner];
-      clipboard->owner->setting_same_owner = FALSE;
-    }
-  else
-    {
-      GtkClipboardOwner *new_owner;
-
-      /* We do not set the new owner on clipboard->owner immediately,
-       * because declareTypes could (but not always does -- see also the
-       * comment at pasteboardChangedOwner above) cause clipboard_unset
-       * to be called, which releases clipboard->owner.
-       */
-      new_owner = [[GtkClipboardOwner alloc] initWithClipboard:clipboard];
-      clipboard->change_count = [clipboard->pasteboard declareTypes: [types allObjects]
-                                                              owner: new_owner];
-
-      /* In case pasteboardChangedOwner was not triggered, check to see
-       * whether the previous owner still needs to be released.
-       */
-      if (clipboard->owner)
-        [clipboard->owner release];
-      clipboard->owner = new_owner;
-    }
-
-  [types release];
-  [pool release];
-
   clipboard->user_data = user_data;
   clipboard->have_owner = have_owner;
   if (have_owner)
@@ -445,18 +371,15 @@ gtk_clipboard_set_contents (GtkClipboard         *clipboard,
     gtk_target_list_unref (clipboard->target_list);
   clipboard->target_list = gtk_target_list_new (targets, n_targets);
 
+  types = _gtk_quartz_target_entries_to_pasteboard_types (targets, n_targets);
+
+  [clipboard->pasteboard declareTypes:[types allObjects] owner:owner];
+  [types release];
+  [pool release];
+
   return TRUE;
 }
 
-/**
- * gtk_clipboard_set_with_data: (skip)
- * @clipboard:
- * @targets: (array length=n_targets):
- * @n_targets:
- * @get_func: (scope async):
- * @clear_func: (scope async):
- * @user_data:
- */
 gboolean
 gtk_clipboard_set_with_data (GtkClipboard          *clipboard,
 			     const GtkTargetEntry  *targets,
@@ -474,14 +397,6 @@ gtk_clipboard_set_with_data (GtkClipboard          *clipboard,
 				     FALSE);
 }
 
-/**
- * gtk_clipboard_set_with_owner: (skip)
- * @clipboard:
- * @targets: (array length=n_targets):
- * @get_func: (scope async):
- * @clear_func: (scope async):
- * @owner:
- */
 gboolean
 gtk_clipboard_set_with_owner (GtkClipboard          *clipboard,
 			      const GtkTargetEntry  *targets,
@@ -500,23 +415,11 @@ gtk_clipboard_set_with_owner (GtkClipboard          *clipboard,
 				     TRUE);
 }
 
-/**
- * gtk_clipboard_get_owner:
- * @clipboard:
- *
- * Returns: (transfer none):
- */
 GObject *
 gtk_clipboard_get_owner (GtkClipboard *clipboard)
 {
   g_return_val_if_fail (clipboard != NULL, NULL);
 
-  if (clipboard->change_count < [clipboard->pasteboard changeCount])
-    {
-      clipboard_unset (clipboard);
-      clipboard->change_count = [clipboard->pasteboard changeCount];
-    }
-
   if (clipboard->have_owner)
     return clipboard->user_data;
   else
@@ -530,12 +433,12 @@ clipboard_unset (GtkClipboard *clipboard)
   gpointer old_data;
   gboolean old_have_owner;
   gint old_n_storable_targets;
-
+  
   old_clear_func = clipboard->clear_func;
   old_data = clipboard->user_data;
   old_have_owner = clipboard->have_owner;
   old_n_storable_targets = clipboard->n_storable_targets;
-
+  
   if (old_have_owner)
     {
       clipboard_remove_owner_notify (clipboard);
@@ -545,14 +448,11 @@ clipboard_unset (GtkClipboard *clipboard)
   clipboard->n_storable_targets = -1;
   g_free (clipboard->storable_targets);
   clipboard->storable_targets = NULL;
-
-  if (clipboard->owner)
-    [clipboard->owner release];
-  clipboard->owner = NULL;
+      
   clipboard->get_func = NULL;
   clipboard->clear_func = NULL;
   clipboard->user_data = NULL;
-
+  
   if (old_clear_func)
     old_clear_func (clipboard, old_data);
 
@@ -573,20 +473,10 @@ clipboard_unset (GtkClipboard *clipboard)
 void
 gtk_clipboard_clear (GtkClipboard *clipboard)
 {
-  clipboard_unset (clipboard);
-#if MAC_OS_X_VERSION_MAX_ALLOWED >= 1060
-  if (gdk_quartz_osx_version() >= GDK_OSX_SNOW_LEOPARD)
-    [clipboard->pasteboard clearContents];
-#if MAC_OS_X_VERSION_MIN_REQUIRED < 1060
-  else
-#endif
-#endif
-#if MAC_OS_X_VERSION_MIN_REQUIRED < 1060
-    [clipboard->pasteboard declareTypes:nil owner:nil];
-#endif
+  [clipboard->pasteboard declareTypes:nil owner:nil];
 }
 
-static void
+static void 
 text_get_func (GtkClipboard     *clipboard,
 	       GtkSelectionData *selection_data,
 	       guint             info,
@@ -595,14 +485,14 @@ text_get_func (GtkClipboard     *clipboard,
   gtk_selection_data_set_text (selection_data, data, -1);
 }
 
-static void
+static void 
 text_clear_func (GtkClipboard *clipboard,
 		 gpointer      data)
 {
   g_free (data);
 }
 
-void
+void 
 gtk_clipboard_set_text (GtkClipboard *clipboard,
 			const gchar  *text,
 			gint          len)
@@ -611,11 +501,11 @@ gtk_clipboard_set_text (GtkClipboard *clipboard,
 
   g_return_if_fail (clipboard != NULL);
   g_return_if_fail (text != NULL);
-
+  
   if (len < 0)
     len = strlen (text);
-
-  gtk_clipboard_set_with_data (clipboard,
+  
+  gtk_clipboard_set_with_data (clipboard, 
 			       &target, 1,
 			       text_get_func, text_clear_func,
 			       g_strndup (text, len));
@@ -623,7 +513,7 @@ gtk_clipboard_set_text (GtkClipboard *clipboard,
 }
 
 
-static void
+static void 
 pixbuf_get_func (GtkClipboard     *clipboard,
 		 GtkSelectionData *selection_data,
 		 guint             info,
@@ -632,7 +522,7 @@ pixbuf_get_func (GtkClipboard     *clipboard,
   gtk_selection_data_set_pixbuf (selection_data, data);
 }
 
-static void
+static void 
 pixbuf_clear_func (GtkClipboard *clipboard,
 		   gpointer      data)
 {
@@ -662,7 +552,7 @@ gtk_clipboard_set_image (GtkClipboard *clipboard,
       targets[i].target = gdk_atom_name (pair->target);
     }
 
-  gtk_clipboard_set_with_data (clipboard,
+  gtk_clipboard_set_with_data (clipboard, 
 			       targets, n_targets,
 			       pixbuf_get_func, pixbuf_clear_func,
 			       g_object_ref (pixbuf));
@@ -674,14 +564,7 @@ gtk_clipboard_set_image (GtkClipboard *clipboard,
   gtk_target_list_unref (list);
 }
 
-/**
- * gtk_clipboard_request_contents:
- * @clipboard:
- * @target:
- * @callback: (scope async):
- * @user_data:
- */
-void
+void 
 gtk_clipboard_request_contents (GtkClipboard            *clipboard,
 				GdkAtom                  target,
 				GtkClipboardReceivedFunc callback,
@@ -696,13 +579,7 @@ gtk_clipboard_request_contents (GtkClipboard            *clipboard,
   gtk_selection_data_free (data);
 }
 
-/**
- * gtk_clipboard_request_text:
- * @clipboard:
- * @callback: (scope async):
- * @user_data:
- */
-void
+void 
 gtk_clipboard_request_text (GtkClipboard                *clipboard,
 			    GtkClipboardTextReceivedFunc callback,
 			    gpointer                     user_data)
@@ -714,13 +591,6 @@ gtk_clipboard_request_text (GtkClipboard                *clipboard,
   g_free (data);
 }
 
-/**
- * gtk_clipboard_request_rich_text:
- * @clipboard:
- * @buffer:
- * @callback: (scope async):
- * @user_data:
- */
 void
 gtk_clipboard_request_rich_text (GtkClipboard                    *clipboard,
                                  GtkTextBuffer                   *buffer,
@@ -731,15 +601,6 @@ gtk_clipboard_request_rich_text (GtkClipboard                    *clipboard,
 }
 
 
-/**
- * gtk_clipboard_wait_for_rich_text:
- * @clipboard:
- * @buffer:
- * @format: (out):
- * @length: (out):
- *
- * Returns: (nullable):
- */
 guint8 *
 gtk_clipboard_wait_for_rich_text (GtkClipboard  *clipboard,
                                   GtkTextBuffer *buffer,
@@ -750,13 +611,7 @@ gtk_clipboard_wait_for_rich_text (GtkClipboard  *clipboard,
   return NULL;
 }
 
-/**
- * gtk_clipboard_request_image:
- * @clipboard:
- * @callback: (scope async):
- * @user_data:
- */
-void
+void 
 gtk_clipboard_request_image (GtkClipboard                  *clipboard,
 			     GtkClipboardImageReceivedFunc  callback,
 			     gpointer                       user_data)
@@ -769,13 +624,7 @@ gtk_clipboard_request_image (GtkClipboard                  *clipboard,
     g_object_unref (pixbuf);
 }
 
-/**
- * gtk_clipboard_request_uris:
- * @clipboard:
- * @callback: (scope async):
- * @user_data:
- */
-void
+void 
 gtk_clipboard_request_uris (GtkClipboard                *clipboard,
 			    GtkClipboardURIReceivedFunc  callback,
 			    gpointer                     user_data)
@@ -787,13 +636,7 @@ gtk_clipboard_request_uris (GtkClipboard                *clipboard,
   g_strfreev (uris);
 }
 
-/**
- * gtk_clipboard_request_targets:
- * @clipboard:
- * @callback: (scope async):
- * @user_data:
- */
-void
+void 
 gtk_clipboard_request_targets (GtkClipboard                *clipboard,
 			       GtkClipboardTargetsReceivedFunc callback,
 			       gpointer                     user_data)
@@ -806,13 +649,6 @@ gtk_clipboard_request_targets (GtkClipboard                *clipboard,
   callback (clipboard, targets, n_targets, user_data);
 }
 
-/**
- * gtk_clipboard_wait_for_contents:
- * @clipboard:
- * @target:
- *
- * Returns: (nullable):
- */
 GtkSelectionData *
 gtk_clipboard_wait_for_contents (GtkClipboard *clipboard,
 				 GdkAtom       target)
@@ -820,13 +656,7 @@ gtk_clipboard_wait_for_contents (GtkClipboard *clipboard,
   NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
   GtkSelectionData *selection_data = NULL;
 
-  if (clipboard->change_count < [clipboard->pasteboard changeCount])
-    {
-      clipboard_unset (clipboard);
-      clipboard->change_count = [clipboard->pasteboard changeCount];
-    }
-
-  if (target == gdk_atom_intern_static_string ("TARGETS"))
+  if (target == gdk_atom_intern_static_string ("TARGETS")) 
     {
       NSArray *types = [clipboard->pasteboard types];
       int i, length;
@@ -834,12 +664,10 @@ gtk_clipboard_wait_for_contents (GtkClipboard *clipboard,
       GdkAtom *atoms;
 
       length = [types count] * sizeof (GdkAtom);
-
+      
       selection_data = g_slice_new0 (GtkSelectionData);
       selection_data->selection = clipboard->selection;
       selection_data->target = target;
-      if (!selection_data->display)
-	selection_data->display = gdk_display_get_default ();
 
       atoms = g_malloc (length);
 
@@ -866,19 +694,13 @@ gtk_clipboard_wait_for_contents (GtkClipboard *clipboard,
   return selection_data;
 }
 
-/**
- * gtk_clipboard_wait_for_text:
- * @clipboard:
- *
- * Returns: (nullable):
- */
 gchar *
 gtk_clipboard_wait_for_text (GtkClipboard *clipboard)
 {
   GtkSelectionData *data;
   gchar *result;
 
-  data = gtk_clipboard_wait_for_contents (clipboard,
+  data = gtk_clipboard_wait_for_contents (clipboard, 
 					  gdk_atom_intern_static_string ("UTF8_STRING"));
 
   result = (gchar *)gtk_selection_data_get_text (data);
@@ -888,36 +710,30 @@ gtk_clipboard_wait_for_text (GtkClipboard *clipboard)
   return result;
 }
 
-/**
- * gtk_clipboard_wait_for_image:
- * @clipboard:
- *
- * Returns: (nullable) (transfer full):
- */
 GdkPixbuf *
 gtk_clipboard_wait_for_image (GtkClipboard *clipboard)
 {
-  GdkAtom target = gdk_atom_intern_static_string("image/tiff");
+  const gchar *priority[] = { "image/png", "image/tiff", "image/jpeg", "image/gif", "image/bmp" };
+  int i;
   GtkSelectionData *data;
 
-  data = gtk_clipboard_wait_for_contents (clipboard, target);
+  for (i = 0; i < G_N_ELEMENTS (priority); i++) 
+    {    
+      data = gtk_clipboard_wait_for_contents (clipboard, gdk_atom_intern_static_string (priority[i]));
 
-  if (data && data->data)
-    {
-      GdkPixbuf *pixbuf = gtk_selection_data_get_pixbuf (data);
-      gtk_selection_data_free (data);
-      return pixbuf;
-    }
+      if (data)
+	{
+	  GdkPixbuf *pixbuf = gtk_selection_data_get_pixbuf (data);
+
+	  gtk_selection_data_free (data);
+
+	  return pixbuf;
+	}  
+  }
 
   return NULL;
 }
 
-/**
- * gtk_clipboard_wait_for_uris:
- * @clipboard:
- *
- * Returns: (nullable) (array zero-terminated=1) (transfer full) (element-type utf8):
- */
 gchar **
 gtk_clipboard_wait_for_uris (GtkClipboard *clipboard)
 {
@@ -932,17 +748,11 @@ gtk_clipboard_wait_for_uris (GtkClipboard *clipboard)
       gtk_selection_data_free (data);
 
       return uris;
-    }
+    }  
 
   return NULL;
 }
 
-/**
- * gtk_clipboard_get_display:
- * @clipboard:
- *
- * Returns: (transfer none):
- */
 GdkDisplay *
 gtk_clipboard_get_display (GtkClipboard *clipboard)
 {
@@ -993,7 +803,7 @@ gtk_clipboard_wait_is_image_available (GtkClipboard *clipboard)
   GtkSelectionData *data;
   gboolean result = FALSE;
 
-  data = gtk_clipboard_wait_for_contents (clipboard,
+  data = gtk_clipboard_wait_for_contents (clipboard, 
 					  gdk_atom_intern_static_string ("TARGETS"));
   if (data)
     {
@@ -1010,7 +820,7 @@ gtk_clipboard_wait_is_uris_available (GtkClipboard *clipboard)
   GtkSelectionData *data;
   gboolean result = FALSE;
 
-  data = gtk_clipboard_wait_for_contents (clipboard,
+  data = gtk_clipboard_wait_for_contents (clipboard, 
 					  gdk_atom_intern_static_string ("TARGETS"));
   if (data)
     {
@@ -1021,33 +831,14 @@ gtk_clipboard_wait_is_uris_available (GtkClipboard *clipboard)
   return result;
 }
 
-/**
- * gtk_clipboard_wait_for_targets:
- * @clipboard: a #GtkClipboard
- * @targets: (out) (array length=n_targets) (transfer container): location
- *           to store an array of targets. The result stored here must
- *           be freed with g_free().
- * @n_targets: (out): location to store number of items in @targets.
- *
- * Returns a list of targets that are present on the clipboard, or %NULL
- * if there aren’t any targets available. The returned list must be
- * freed with g_free().
- * This function waits for the data to be received using the main
- * loop, so events, timeouts, etc, may be dispatched during the wait.
- *
- * Returns: %TRUE if any targets are present on the clipboard,
- *               otherwise %FALSE.
- *
- * Since: 2.4
- */
 gboolean
-gtk_clipboard_wait_for_targets (GtkClipboard  *clipboard,
+gtk_clipboard_wait_for_targets (GtkClipboard  *clipboard, 
 				GdkAtom      **targets,
 				gint          *n_targets)
 {
   GtkSelectionData *data;
   gboolean result = FALSE;
-
+  
   g_return_val_if_fail (clipboard != NULL, FALSE);
 
   /* If the display supports change notification we cache targets */
@@ -1056,19 +847,19 @@ gtk_clipboard_wait_for_targets (GtkClipboard  *clipboard,
     {
       if (n_targets)
  	*n_targets = clipboard->n_cached_targets;
-
+ 
       if (targets)
  	*targets = g_memdup (clipboard->cached_targets,
  			     clipboard->n_cached_targets * sizeof (GdkAtom));
 
        return TRUE;
     }
-
+  
   if (n_targets)
     *n_targets = 0;
-
+      
   if (targets)
-    *targets = NULL;
+    *targets = NULL;      
 
   data = gtk_clipboard_wait_for_contents (clipboard, gdk_atom_intern_static_string ("TARGETS"));
 
@@ -1076,24 +867,24 @@ gtk_clipboard_wait_for_targets (GtkClipboard  *clipboard,
     {
       GdkAtom *tmp_targets;
       gint tmp_n_targets;
-
+       
       result = gtk_selection_data_get_targets (data, &tmp_targets, &tmp_n_targets);
-
+ 
       if (gdk_display_supports_selection_notification (gtk_clipboard_get_display (clipboard)))
  	{
  	  clipboard->n_cached_targets = tmp_n_targets;
  	  clipboard->cached_targets = g_memdup (tmp_targets,
  						tmp_n_targets * sizeof (GdkAtom));
  	}
-
+ 
       if (n_targets)
  	*n_targets = tmp_n_targets;
-
+ 
       if (targets)
  	*targets = tmp_targets;
       else
  	g_free (tmp_targets);
-
+      
       gtk_selection_data_free (data);
     }
 
@@ -1101,7 +892,7 @@ gtk_clipboard_wait_for_targets (GtkClipboard  *clipboard,
 }
 
 static GtkClipboard *
-clipboard_peek (GdkDisplay *display,
+clipboard_peek (GdkDisplay *display, 
 		GdkAtom     selection,
 		gboolean    only_if_exists)
 {
@@ -1130,13 +921,13 @@ clipboard_peek (GdkDisplay *display,
       NSString *pasteboard_name;
       clipboard = g_object_new (GTK_TYPE_CLIPBOARD, NULL);
 
-      if (selection == GDK_SELECTION_CLIPBOARD)
+      if (selection == GDK_SELECTION_CLIPBOARD) 
 	pasteboard_name = NSGeneralPboard;
-      else
+      else 
 	{
 	  char *atom_string = gdk_atom_name (selection);
 
-	  pasteboard_name = [NSString stringWithFormat:@"_GTK_%@",
+	  pasteboard_name = [NSString stringWithFormat:@"_GTK_%@", 
 			     [NSString stringWithUTF8String:atom_string]];
 	  g_free (atom_string);
 	}
@@ -1155,7 +946,7 @@ clipboard_peek (GdkDisplay *display,
 			G_CALLBACK (clipboard_display_closed), clipboard);
       gdk_display_request_selection_notification (display, selection);
     }
-
+  
   return clipboard;
 }
 
@@ -1177,7 +968,7 @@ gtk_clipboard_wait_is_target_available (GtkClipboard *clipboard,
   GdkAtom *targets;
   gint i, n_targets;
   gboolean retval = FALSE;
-
+    
   if (!gtk_clipboard_wait_for_targets (clipboard, &targets, &n_targets))
     return FALSE;
 
@@ -1191,21 +982,15 @@ gtk_clipboard_wait_is_target_available (GtkClipboard *clipboard,
     }
 
   g_free (targets);
-
+  
   return retval;
 }
 
-void
+void 
 _gtk_clipboard_handle_event (GdkEventOwnerChange *event)
 {
 }
 
-/**
- * gtk_clipboard_set_can_store:
- * @clipboard:
- * @targets: (allow-none) (array length=n_targets):
- * @n_targets:
- */
 void
 gtk_clipboard_set_can_store (GtkClipboard         *clipboard,
  			     const GtkTargetEntry *targets,
@@ -1217,90 +1002,11 @@ gtk_clipboard_set_can_store (GtkClipboard         *clipboard,
 void
 gtk_clipboard_store (GtkClipboard *clipboard)
 {
-  int i;
-  int n_targets = 0;
-  GtkTargetEntry *targets;
-
-  g_return_if_fail (GTK_IS_CLIPBOARD (clipboard));
-
-  if (!clipboard->target_list || !clipboard->get_func)
-    return;
-
-  /* We simply store all targets into the OS X clipboard. We should be
-   * using the functions gdk_display_supports_clipboard_persistence() and
-   * gdk_display_store_clipboard(), but since for OS X the clipboard support
-   * was implemented in GTK+ and not through GdkSelections, we do it this
-   * way. Doing this properly could be worthwhile to implement in the future.
-   */
-
-  targets = gtk_target_table_new_from_list (clipboard->target_list,
-                                            &n_targets);
-  for (i = 0; i < n_targets; i++)
-    {
-      GtkSelectionData selection_data;
-
-      /* in each loop iteration, check if the content is still
-       * there, because calling get_func() can do anything to
-       * the clipboard
-       */
-      if (!clipboard->target_list || !clipboard->get_func)
-        break;
-
-      memset (&selection_data, 0, sizeof (GtkSelectionData));
-
-      selection_data.selection = clipboard->selection;
-      selection_data.target = gdk_atom_intern_static_string (targets[i].target);
-      selection_data.display = gdk_display_get_default ();
-      selection_data.length = -1;
-
-      clipboard->get_func (clipboard, &selection_data,
-                           targets[i].info, clipboard->user_data);
-
-      if (selection_data.length >= 0)
-        _gtk_quartz_set_selection_data_for_pasteboard (clipboard->pasteboard,
-                                                       &selection_data);
-
-      g_free (selection_data.data);
-    }
-
-  if (targets)
-    gtk_target_table_free (targets, n_targets);
+  /* FIXME: Implement */
 }
 
 void
 _gtk_clipboard_store_all (void)
 {
-  GtkClipboard *clipboard;
-  GSList *displays, *list;
-
-  displays = gdk_display_manager_list_displays (gdk_display_manager_get ());
-
-  list = displays;
-  while (list)
-    {
-      GdkDisplay *display = list->data;
-
-      clipboard = clipboard_peek (display, GDK_SELECTION_CLIPBOARD, TRUE);
-
-      if (clipboard)
-        gtk_clipboard_store (clipboard);
-
-      list = list->next;
-    }
-}
-
-/**
- * gtk_clipboard_get_selection:
- * @clipboard:
- *
- * Returns: the selection
- *
- * Since: 3.22
- */
-GdkAtom
-gtk_clipboard_get_selection (GtkClipboard *clipboard)
-{
-  g_return_val_if_fail (GTK_IS_CLIPBOARD (clipboard), GDK_NONE);
-
-  return clipboard->selection;
+  /* FIXME: Implement */
 }
